package me.totalfreedom.totalfreedommod.perms;

import net.luckperms.api.model.group.Group;
import net.luckperms.api.model.group.GroupManager;

import java.util.Set;
import java.util.concurrent.CompletableFuture;

public class GroupWrapper
{
    private final GroupManager manager;
    private final Set<Group> groupSet;

    public GroupWrapper(GroupManager manager)
    {
        this.manager = manager;
        this.groupSet = manager.getLoadedGroups();
    }

    public CompletableFuture<Group> getGroup(String name)
    {
        if (!manager.isLoaded(name) || !groupSet.contains(manager.getGroup(name)))
        {
            return manager.createAndLoadGroup(name);
        }

        return CompletableFuture.supplyAsync(() -> manager.getGroup(name));
    }

    public CompletableFuture<Void> removeGroup(String name)
    {
        return getGroup(name).thenAccept(a ->
        {
            manager.deleteGroup(a);
            groupSet.remove(a);
        });
    }

    public CompletableFuture<Void> addGroup(String name)
    {
        return getGroup(name).thenAccept(groupSet::add);
    }

    public CompletableFuture<Void> saveGroup(String name)
    {
        return getGroup(name).thenAccept(manager::saveGroup);
    }
}
