package me.totalfreedom.totalfreedommod.perms;

import net.luckperms.api.cacheddata.CachedMetaData;
import net.luckperms.api.cacheddata.CachedPermissionData;
import net.luckperms.api.context.ImmutableContextSet;
import net.luckperms.api.model.user.User;
import net.luckperms.api.model.user.UserManager;
import net.luckperms.api.platform.PlayerAdapter;
import net.luckperms.api.query.QueryOptions;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

public class UserWrapper
{
    private final UserManager manager;
    private final PlayerAdapter<OfflinePlayer> playerAdapter;
    private final Set<User> userSet;
    private final Set<UserData> userDataSet;

    public UserWrapper(UserManager manager, PlayerAdapter<OfflinePlayer> adapter)
    {
        this.manager = manager;
        this.userSet = manager.getLoadedUsers();
        this.playerAdapter = adapter;
        this.userDataSet = new HashSet<>();
    }

    public CompletableFuture<Void> addUserData(OfflinePlayer player)
    {
        User user = playerAdapter.getUser(player);
        CachedPermissionData permissionData = playerAdapter.getPermissionData(player);
        ImmutableContextSet contextSet = playerAdapter.getContext(player);
        QueryOptions queryOptions = playerAdapter.getQueryOptions(player);
        CachedMetaData metaData = playerAdapter.getMetaData(player);

        UserData userData = new UserData(user, metaData, permissionData, contextSet, queryOptions);
        return CompletableFuture.runAsync(() -> userDataSet.add(userData));
    }

    public CompletableFuture<User> getUser(OfflinePlayer player)
    {
        if (!manager.isLoaded(player.getUniqueId()) || !userSet.contains(manager.getUser(player.getUniqueId())))
        {
            return manager.loadUser(player.getUniqueId());
        }

        return CompletableFuture.supplyAsync(() -> manager.getUser(player.getUniqueId()));
    }

    public CompletableFuture<Void> removeUser(OfflinePlayer player)
    {
        return getUser(player).thenAccept(a ->
        {
            manager.cleanupUser(a);
            userSet.remove(a);
        });
    }

    public CompletableFuture<UserData> getUserData(OfflinePlayer player)
    {
        return getUser(player).thenApply(user ->
        {
            for (UserData userData : userDataSet)
            {
                if (userData.getUser().getUniqueId().equals(user.getUniqueId()))
                {
                    return userData;
                }
            }
            return null;
        });
    }

    public CompletableFuture<Void> removeUserData(OfflinePlayer player)
    {
        return getUserData(player).thenAccept(userDataSet::remove);
    }

    public CompletableFuture<Void> updateUserData(OfflinePlayer player)
    {
        return getUserData(player).thenApply(userData ->
        {
            UserData newData = userData.clone();
            CachedPermissionData permissionData = playerAdapter.getPermissionData(player);
            ImmutableContextSet contextSet = playerAdapter.getContext(player);
            QueryOptions queryOptions = playerAdapter.getQueryOptions(player);
            CachedMetaData metaData = playerAdapter.getMetaData(player);

            newData.setPermissionData(permissionData);
            newData.setContextSet(contextSet);
            newData.setQueryOptions(queryOptions);
            newData.setMetaData(metaData);

            userDataSet.remove(userData);
            return newData;
        }).thenAccept(userDataSet::add);
    }

    public CompletableFuture<Void> saveUser(User user)
    {
        return CompletableFuture.runAsync(() -> manager.saveUser(user));
    }
}
